// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using System;
using System.Linq.Expressions;
using JetBrains.Annotations;
using Microsoft.EntityFrameworkCore.Storage;

namespace Microsoft.EntityFrameworkCore.Metadata.Internal
{
    /// <summary>
    ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
    ///     directly from your code. This API may change or be removed in future releases.
    /// </summary>
    public class ContextParameterBinding : ServiceParameterBinding
    {
        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        public ContextParameterBinding(
            [NotNull] Type contextType,
            [CanBeNull] IPropertyBase consumedProperty = null)
            : base(contextType, contextType, consumedProperty)
        {
        }

        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        public override Expression BindToParameter(
            Expression materializationExpression,
            Expression entityTypeExpression,
            Expression entityExpression)
        {
            var propertyExpression
                = Expression.Property(
                    materializationExpression,
                    MaterializationContext.ContextProperty);

            return ServiceType != typeof(DbContext)
                ? (Expression)Expression.TypeAs(propertyExpression, ServiceType)
                : propertyExpression;
        }
    }
}
