// Copyright (c) .NET Foundation. All rights reserved.
// Licensed under the Apache License, Version 2.0. See License.txt in the project root for license information.

using System;
using System.Linq.Expressions;
using JetBrains.Annotations;
using Microsoft.EntityFrameworkCore.Storage;

namespace Microsoft.EntityFrameworkCore.Metadata.Internal
{
    /// <summary>
    ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
    ///     directly from your code. This API may change or be removed in future releases.
    /// </summary>
    public interface IEntityMaterializerSource
    {
        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        Expression CreateReadValueExpression(
            [NotNull] Expression valueBuffer,
            [NotNull] Type type,
            int index,
            [CanBeNull] IPropertyBase property);

        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        [Obsolete("Use CreateReadValueExpression making sure to pass bound property if available.")]
        Expression CreateReadValueCallExpression([NotNull] Expression valueBuffer, int index);

        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        Expression CreateMaterializeExpression(
            [NotNull] IEntityType entityType,
            [NotNull] Expression materializationExpression,
            [CanBeNull] int[] indexMap = null);

        /// <summary>
        ///     This API supports the Entity Framework Core infrastructure and is not intended to be used
        ///     directly from your code. This API may change or be removed in future releases.
        /// </summary>
        Func<MaterializationContext, object> GetMaterializer([NotNull] IEntityType entityType);
    }
}
